#property copyright "Copyright 2008, Ralph Ronnquist"

#import "PipsChartData.ex4"
int pcBars(double &chart[][]);
datetime pcTime(double &chart[][],int shift);
int pcCount(double &chart[][],int shift);
double pcOpen(double &chart[][],int shift);
double pcHigh(double &chart[][],int shift);
double pcLow(double &chart[][],int shift);
double pcClose(double &chart[][],int shift);

void PipsChartReset(double &chart[][],int pips,int bars,bool box,bool center,bool stretch);
bool PipsChartUpdate(double &chart[][],int pips,datetime t,double p);
void PipsChartShiftOut(double &data[]);
void PipsChartSave(double &chart[][],string filename);
void PipsChartLoad(double &chart[][],string filename);
#import

//************************************************************
// Setup for PipsChart
extern int pips = 30;
int count = 0;
bool box = true;
bool center = true;
bool stretch = true;

double chart[][6];

void PipsChartInit()
{

    ArrayResize( chart, 10 );
    PipsChartReset( chart, pips, count, box, center, stretch );

    // Draw three PipsChart bars on the main chart
    for ( int i = 3; i >= 0; i-- ) {
        if ( pcTime( chart, i ) != 0 )
            showBar( chart, pips, i );
    }
}

//************************************************************
// Decision logic
extern double wick = 0.8; // Required wick length as a fraction of "pips"

#define OP_NONE -1

// Returns OP_NONE, OP_BUY or OP_SELL
int pcCheck()
{
    // Capture price move
    PipsChartUpdate( chart, pips, TimeCurrent(), Bid );
    showBar( chart, pips, 0 );

    // Filter for only once per PipsChart bar
    static datetime last = 0;
    if ( pcTime( chart, 0 ) == last )
        return( OP_NONE );
    last = pcTime( chart, 0 );

    // check for buy pattern: previous bar is a bull with a sizable down wick
    if ( pcClose( chart, 1 ) > pcOpen( chart, 1 ) ) {
        if ( pcOpen( chart, 1 ) - pcLow( chart, 1 ) >= wick * pips * Point )
            return( OP_BUY );
        return( OP_NONE );
    }
    
    // check for sell pattern: previous bar is a bear with a sizable up wick
    if ( pcClose( chart, 1 ) < pcOpen( chart, 1 ) ) {
        if ( pcHigh( chart, 1 ) - pcOpen( chart, 1 ) >= wick * pips * Point )
            return( OP_SELL );
        return( OP_NONE );
    }

    return( OP_NONE );
}

///////////////////////////////
color colors[] = { Blue, Red, Gray, Gray };

void showBar(double & chart[][],int pips,int bar)
{
    datetime past = pcTime( chart, bar );
    datetime now = Time[0];
    if ( bar > 0 )
        now = pcTime( chart, bar-1 );
    double hi = pcHigh( chart, bar );
    double hx = MathMax( pcOpen( chart, bar ), pcClose( chart, bar ) );
    double lx = MathMin( pcOpen( chart, bar ), pcClose( chart, bar ) );
    double lo = pcLow( chart, bar );
    drawRectangle( "head " + past + pips, past, hi, now, hx, colors[2] );
    drawRectangle( "body " + past + pips, past, hx, now, lx, 
        colors[ 0 + ( pcOpen( chart, bar ) > pcClose( chart, bar ) ) ] );
    drawRectangle( "legs " + past + pips, past, lx, now, lo, colors[3] );
}

void drawRectangle(string name,datetime t0,double p0,datetime t1,double p1,color c)
{
    if ( ObjectFind( name ) < 0 ) {
        ObjectCreate( name, OBJ_RECTANGLE, 0, t0, p0, t1, p1 );
    } else {
        ObjectMove( name, 0, t0, p0 );
        ObjectMove( name, 1, t1, p1 );
    }
    ObjectSet( name, OBJPROP_COLOR, c );
    ObjectSet( name, OBJPROP_BACK, true );
}

